#!/bin/bash
# -cr-beg-
# #############################################################################
#   Licensed Materials - Property of IBM
#
#   "Restricted Materials of IBM"
#
#   5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
#
#   (C) Copyright IBM Corp. 2011, 2021 All Rights Reserved
#
#   (C) Copyright 2016, 2021 UNICOM (R) Systems, Inc. - All Rights Reserved
#
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
# #############################################################################
# -cr-end-
# This script creates symbolic links for the ODPP and ICU libraries used by ODPP
# Usage:
# sh createODPPsymboliclinks.sh <path> <major version> <full version> <hybrid version>
# <path> - This is the path to the folder containing the ODPP binaries.
# <major version> - This is two position ODPP version (i.e., v.r)
# <full version> - This is the full ODPP version (i.e., v.r.m.f)
# <hybrid version> - This is a hybrid two position ODPP version (i.e., vr)
# <cleanonly> - Remove symbolic links, do not create new ones

if [ "$1" = "" ]
then
   echo "The 1st argument to the script must be the folder containing the ODPP binaries."
   exit 1
fi
if [ "$2" = "" ]
then
   echo "The 2nd argument to the script must be the major version (e.g. 11.3)."
   exit 1
fi
if [ "$3" = "" ]
then
   echo "The 3rd argument to the script must be the full version (e.g. 11.3.0.5)."
   exit 1
fi
if [ "$4" = "" ]
then
   echo "The 4th argument to the script must be the version hybrid (e.g. 113)."
   exit 1
fi
if [ "$5" = "cleanonly" ]
then
   echo "The option cleanonly specified. Symlinks will only be removed."
   cleanonly=1
else
   cleanonly=0
fi

majorver=$2
fullversion=$3
hybridver=$4

platform=`uname -s`
_MAJOR_RELEASE=`uname -a`

echo "Detecting OS..."
case ${platform} in
    AIX)
        _OS="AIX"
        _DISTRO="IBM"
        soname="a"
        _ARCH=`uname -p`
        platform_version=`uname -v`
        platform_level=`uname -r`
        _MAJOR_RELEASE="${platform_version}.${platform_level}"
        ;;

    HP-UX)
        _OS="HP-UX"
        _DISTRO="HP"
        platform_version=`uname -m`
        _MAJOR_RELEASE=`uname -r`
        if [ "$platform_version" = "ia64" ]; then
            soname="so"
            _ARCH="ia64"
        else
            soname="sl"
            _ARCH="PA-RISC"
        fi
        ;;

    Linux)
        _OS="Linux"
        _ARCH=`uname -p`
        soname="so"
        if [ -f /etc/redhat-release ] ; then
            _DISTRO="RedHat"
            _OS_VERSION=`cat /proc/version`

            # Test if this is CentOS
            _TEMP_OS=`cat /etc/redhat-release | grep 'CentOS'`
            # If this is a CentOS, RC = 0
            _RC_CENTOS=$?
            # Test if this is some other RedHat Linux
            _TEMP_OS=`cat /etc/redhat-release | grep 'Hat'`
            # If this is Red Hat, RC = 0
            _RC_REDHAT=$?
            # Test if this is RedHat Enterprise Linux
            _TEMP_OS=`cat /etc/redhat-release | grep 'Hat Enterprise'`
            # If this is Red Hat Enterprise, RC = 0
            _RC_RHEL=$?
            _TEMP_OS=`cat /etc/issue.net | head -1 | grep 'Oracle'`
            # If this is Oracle Enterprise Linux, RC = 0
            _RC_OEL=$?
            # If this is OEL, RC = 0

            _TEMP_OS=`uname -p | grep 's390x'`
            # If this is 64 bit zLinux, RC = 0
            _RC_S390X=$?
            # If this is zLinux 64bit, RC = 0
            _TEMP_OS=`uname -p | grep 's390'`
            # If this is 64 bit zLinux, RC = 0
            _RC_S390=$?
            # If this is zLinux 31bit, RC = 0

            if [ "$_RC_CENTOS" = "0" ] ; then
               _DISTRO_NAME="CentOS Linux"
            elif [ "$_RC_S390X" = "0" ] ; then
               _DISTRO_NAME="IBM zLinux s390x"
            elif [ "$_RC_S390" = "0" ] ; then
               _DISTRO_NAME="IBM zLinux s390"
            elif [ "$_RC_OEL" = "0" ] ; then
               _DISTRO_NAME="Oracle Enterprise Linux"
            elif [ "$_RC_RHEL" = "0" ] ; then
               _DISTRO_NAME="Red Hat Enterprise Linux"
            elif [ "$_RC_REDHAT" = "0" ] ; then
               _DISTRO_NAME="Red Hat Linux"
            else
               _DISTRO_NAME="Red Hat (variant) Linux"
            fi

            if grep -q -i "release 4" /etc/redhat-release ; then
               _MAJOR_RELEASE=4
            elif grep -q -i "release 5" /etc/redhat-release ; then
               _MAJOR_RELEASE=5
            elif grep -q -i "release 6" /etc/redhat-release ; then
               _MAJOR_RELEASE=6
            elif grep -q -i "release 7" /etc/redhat-release ; then
               _MAJOR_RELEASE=7
            elif grep -q -i "release 8" /etc/redhat-release ; then
               _MAJOR_RELEASE=8
            elif grep -q -i "release 9" /etc/redhat-release ; then
               _MAJOR_RELEASE=9
            fi
         elif [ -f /etc/SuSE-release ] ; then
            if [ "${_ARCH}" = "s390" -o "${_ARCH}" = "s390x" ]; then
                _DISTRO="IBM z/SuSE"
            else
                _DISTRO="SuSE"
                _DISTRO_NAME="Novell SuSE Linux"
            fi
         elif [ -f /etc/mandrake-release ] ; then
            _DISTRO='Mandrake'
            _DISTRO_NAME="Mandrake Linux"
         fi
         ;;

    SunOS)
        _OS="Solaris"
        soname="so"
        _ARCH=`uname -p`
        _MAJOR_RELEASE=`uname -r`
        _DISTRO="SUN"
     ;;
esac

echo "OS               = ${_OS}"
echo "Vendor           = ${_DISTRO}"
echo "Architecture     = ${_ARCH}"
echo "Release          = ${_MAJOR_RELEASE}"
echo "Library type     = ${soname}"
echo "Target Directory = $1"
echo "Optim version    = ${fullversion}"
echo "hybrid version   = ${hybridver}"

# We will always remove the symbolic link if it exists
# In ODPPFiles, there are the following: 
# The ODPP-Column Map Provider Interface Name is libODPPro${hybridver}
# The ODPP-Column Map Parameter Adapter Interface Name is libODPAdp${hybridver}
# The ODPP-Column Map Parser Interface Name is libODPPsr${hybridver}
# The ODPP-Column Map Hash Interface Name libioqhash${hybridver}

ODPPFILES=(libioqdb2udf libioqhash libioqldsoralib libIOQMODJNI libioqoraudf libioqspe libODPAGEPrvLib libODPCCNPrvLib libODPCDSWAPPrvLib libODPCOLPrvLib libODPDDSWAPPrvLib libODPDSWAPPrvLib libODPEMLPrvLib libODPHASHPrvLib libODPLKPPrvLib libODPNIDPrvLib libODPPAdapter libODPPConvert libODPPDataCnvLib libODPPEnvLib libODPPErrorHandler libODPPICULib libODPPInstMgr libODPPJNI libODPPLDSDB2Lib libODPPLicl libODPPLoader libODPPORAUdf libODPPOSLibUnix libODPPParser libODPProvider libODPPXMLReaderLib libODPPro${hybridver} libODPAdp${hybridver} libODPPsr${hybridver} libioqhash${hybridver})
echo "Deleting symbolic links and duplicates for the ODPP libraries for ${fullversion}."

for odppfile in ${ODPPFILES[@]}
do
   if [ "$platform" = "AIX" ]
   then
      if [[ -L $1/${odppfile}.${soname} || -e $1/${odppfile}.${soname} ]]; then rm $1/${odppfile}.${soname}; fi
      if [[ -L $1/${odppfile}.$majorver.${soname} || -e $1/${odppfile}.$majorver.${soname} ]]; then rm $1/${odppfile}.$majorver.${soname}; fi
   else
      if [[ -L $1/${odppfile}.${soname} || -e $1/${odppfile}.${soname} ]]; then rm $1/${odppfile}.${soname}; fi
      if [[ -L $1/${odppfile}.${soname}.$majorver || -e $1/${odppfile}.${soname}.$majorver ]]; then rm $1/${odppfile}.${soname}.$majorver; fi
      if [[ -L $1/${odppfile}.$majorver.${soname} || -e $1/${odppfile}.$majorver.${soname} ]]; then rm $1/${odppfile}.$majorver.${soname}; fi
   fi
done

if [ "$platform" = "AIX" ]
then
   # For the Db2 UDF library, a .so is used
   if [[ -L $1/libioqdb2udf.so || -e $1/libioqdb2udf.so ]]; then rm $1/libioqdb2udf.so; fi
   if [[ -L $1/libioqdb2udf.so.$majorver || -e $1/libioqdb2udf.so.$majorver ]]; then rm $1/libioqdb2udf.so.$majorver; fi
fi

#Xerces - Starting with 11.3.0.4 iFix12
#We will always remove the symbolic link if it exists
echo "Deleting symbolic links and duplicates for the XERCESC libraries for ${fullversion}."
if [ "$platform" != "AIX" ]
then
   if [[ -L $1/libxerces-c.${soname} || -e $1/libxerces-c.${soname} ]]
   then
       rm $1/libxerces-c.${soname}
   fi
fi

# ICU Symbolic Links
# We will always remove the symbolic link if it exists
echo "Deleting symbolic links and duplicates for the ICU ${icuversion} libraries for ${fullversion}."
icuversion=52
icurelease=2
ICUFILES=(libicudata libicuuc)
ICUVERSIONS=(48 ${icuversion})

for ver in ${ICUVERSIONS[@]}
do
   for icufile in ${ICUFILES[@]}
   do
      if [ "$platform" = "AIX" ]
      then
         if [[ -L $1/${icufile}${ver}.${soname} || -e $1/${icufile}${ver}.${soname} ]]; then rm $1/${icufile}${ver}.${soname}; fi
         if [[ -L $1/${icufile}.${soname} || -e $1/${icufile}.${soname} ]]; then rm $1/${icufile}.${soname}; fi
      else
         if [[ -L $1/${icufile}.${soname}.${ver} || -e $1/${icufile}.${soname}.${ver} ]]; then rm $1/${icufile}.${soname}.${ver}; fi
         if [[ -L $1/${icufile}.${soname} || -e $1/${icufile}.${soname} ]]; then rm $1/${icufile}.${soname}; fi
      fi
   done
done

# If we are only removing symlinks then we are done
if [ $cleanonly = 1 ]
then
    echo Exiting without creating symlinks.
    exit
fi

echo "Creating symbolic links for the ODPP libraries"
if [ "$platform" = "AIX" ]
then
   ln -s libODPProvider.$fullversion.${soname} $1/libODPProvider.$majorver.${soname}
   if [ -e $1/libODPProvider.${soname}.$fullversion ]
   then 
      ln -s libODPProvider.${soname}.$fullversion $1/libODPProvider.${soname}.$majorver
   fi
# The ODPP-Column Map Provider Interface Name. Also used for UDFs
   ln -s libODPProvider.$fullversion.${soname} $1/libODPPro${hybridver}.${soname}
   ln -s libODPPOSLibUnix.$fullversion.${soname} $1/libODPPOSLibUnix.$majorver.${soname}
   ln -s libODPPEnvLib.$fullversion.${soname} $1/libODPPEnvLib.$majorver.${soname}
   ln -s libODPPICULib.$fullversion.${soname} $1/libODPPICULib.$majorver.${soname}
   ln -s libODPPDataCnvLib.$fullversion.${soname} $1/libODPPDataCnvLib.$majorver.${soname}
   if [ -e $1/libODPPConvert.$fullversion.${soname} ]
   then
     ln -s  libODPPConvert.$fullversion.${soname} $1/libODPPConvert.$majorver.${soname}
   fi
   ln -s libODPCCNPrvLib.$fullversion.${soname} $1/libODPCCNPrvLib.$majorver.${soname}
   ln -s libODPNIDPrvLib.$fullversion.${soname} $1/libODPNIDPrvLib.$majorver.${soname}
   ln -s libODPEMLPrvLib.$fullversion.${soname} $1/libODPEMLPrvLib.$majorver.${soname}
   ln -s libODPCOLPrvLib.$fullversion.${soname} $1/libODPCOLPrvLib.$majorver.${soname}
   ln -s libODPHASHPrvLib.$fullversion.${soname} $1/libODPHASHPrvLib.$majorver.${soname}
   ln -s libODPAGEPrvLib.$fullversion.${soname} $1/libODPAGEPrvLib.$majorver.${soname}
   ln -s libODPPLDSDB2Lib.$fullversion.${soname} $1/libODPPLDSDB2Lib.$majorver.${soname}
   ln -s libODPLKPPrvLib.$fullversion.${soname} $1/libODPLKPPrvLib.$majorver.${soname}
   ln -s libODPPAdapter.$fullversion.${soname} $1/libODPPAdapter.$majorver.${soname}
   # The ODPP-Column Map Parameter Adapter Interface Name
   ln -s libODPPAdapter.$fullversion.${soname} $1/libODPAdp${hybridver}.${soname}
   ln -s libODPPParser.$fullversion.${soname} $1/libODPPParser.$majorver.${soname}
   # The ODPP-Column Map Parser Interface Name
   ln -s libODPPParser.$fullversion.${soname} $1/libODPPsr${hybridver}.${soname}
   ln -s libODPPErrorHandler.$fullversion.${soname} $1/libODPPErrorHandler.$majorver.${soname}
   ln -s libODPPInstMgr.$fullversion.${soname} $1/libODPPInstMgr.$majorver.${soname}
   # Don't create symlinks for *SWAP* libraries if they are not present
   if [ -f $1/libODPDSWAPPrvLib.$fullversion.${soname} ]
   then
        ln -s libODPDSWAPPrvLib.$fullversion.${soname} $1/libODPDSWAPPrvLib.$majorver.${soname}
        ln -s libODPCDSWAPPrvLib.$fullversion.${soname} $1/libODPCDSWAPPrvLib.$majorver.${soname}
        ln -s libODPDDSWAPPrvLib.$fullversion.${soname} $1/libODPDDSWAPPrvLib.$majorver.${soname}
   fi
   ln -s libioqhash.$fullversion.${soname} $1/libioqhash.$majorver.${soname}
   if [ -e $1/libioqspe.$fullversion.${soname} ]
   then
      ln -s libioqspe.$fullversion.${soname} $1/libioqspe.$majorver.${soname}
   fi
   # The ODPP-Column Map Hash Interface Name
   ln -s libioqhash.$fullversion.${soname} $1/libioqhash${hybridver}.${soname}
   ln -s libioqldsoralib.$fullversion.${soname} $1/libioqldsoralib.$majorver.${soname}
   ln -s libODPPLoader.$fullversion.${soname} $1/libODPPLoader.$majorver.${soname}
   if [ -e $1/libODPPORAUdf.$fullversion.${soname} ]
   then
      ln -s  libODPPORAUdf.$fullversion.${soname} $1/libODPPORAUdf.$majorver.${soname}
      ln -s  libODPPORAUdf.$fullversion.${soname} $1/libODPPORAUdf.${soname}
   fi
   if [ -e $1/libioqoraudf.$fullversion.${soname} ]
   then
      ln -s  libioqoraudf.$fullversion.${soname} $1/libioqoraudf.$majorver.${soname}
      ln -s  libioqoraudf.$fullversion.${soname} $1/libioqoraudf.${soname}
   fi
# For the Db2 UDF library, a .so is used
   if [ -e $1/libioqdb2udf.so.$fullversion ]
   then
      ln -s  libioqdb2udf.so.$fullversion $1/libioqdb2udf.so.$majorver;
   fi
   if [ -e $1/libIOQMODJNI.$fullversion.${soname} ]
   then
      ln -s  libIOQMODJNI.$fullversion.${soname} $1/libIOQMODJNI.$majorver.${soname};
      ln -s  libIOQMODJNI.$fullversion.${soname} $1/libIOQMODJNI.${soname}
   fi
else
   ln -s libODPProvider.${soname}.$fullversion $1/libODPProvider.${soname}.$majorver
   # BEGIN: The ODPP-Column Map Provider Interface Name. Also used for UDFs
   ln -s libODPProvider.${soname}.$fullversion $1/libODPProvider.$majorver.${soname}
   ln -s libODPProvider.${soname}.$fullversion $1/libODPPro${hybridver}.${soname}
   # END: The ODPP-Column Map Provider Interface Name. Also used for UDFs
   ln -s libODPPOSLibUnix.${soname}.$fullversion $1/libODPPOSLibUnix.${soname}.$majorver
   ln -s libODPPEnvLib.${soname}.$fullversion $1/libODPPEnvLib.${soname}.$majorver
   ln -s libODPPICULib.${soname}.$fullversion $1/libODPPICULib.${soname}.$majorver
   ln -s libODPPDataCnvLib.${soname}.$fullversion $1/libODPPDataCnvLib.${soname}.$majorver
   ln -s libODPCCNPrvLib.${soname}.$fullversion $1/libODPCCNPrvLib.${soname}.$majorver
   ln -s libODPNIDPrvLib.${soname}.$fullversion $1/libODPNIDPrvLib.${soname}.$majorver
   ln -s libODPEMLPrvLib.${soname}.$fullversion $1/libODPEMLPrvLib.${soname}.$majorver
   ln -s libODPCOLPrvLib.${soname}.$fullversion $1/libODPCOLPrvLib.${soname}.$majorver
   ln -s libODPHASHPrvLib.${soname}.$fullversion $1/libODPHASHPrvLib.${soname}.$majorver
   ln -s libODPAGEPrvLib.${soname}.$fullversion $1/libODPAGEPrvLib.${soname}.$majorver
   ln -s libODPPLDSDB2Lib.${soname}.$fullversion $1/libODPPLDSDB2Lib.${soname}.$majorver
   ln -s libODPLKPPrvLib.${soname}.$fullversion $1/libODPLKPPrvLib.${soname}.$majorver
   ln -s libODPPAdapter.${soname}.$fullversion $1/libODPPAdapter.${soname}.$majorver
   # The ODPP-Column Map Parameter Adapter Interface Name
   ln -s libODPPAdapter.${soname}.$fullversion $1/libODPAdp${hybridver}.${soname}
   ln -s libODPPParser.${soname}.$fullversion $1/libODPPParser.${soname}.$majorver
   # The ODPP-Column Map Parser Interface Name
   ln -s libODPPParser.${soname}.$fullversion  $1/libODPPsr${hybridver}.${soname}
   ln -s libODPPErrorHandler.${soname}.$fullversion $1/libODPPErrorHandler.${soname}.$majorver
   ln -s libODPPInstMgr.${soname}.$fullversion $1/libODPPInstMgr.${soname}.$majorver
   # Don't create symlinks for *SWAP* libraries if they are not present
   if [ -f $1/libODPDSWAPPrvLib.${soname}.$fullversion ]
   then
        ln -s libODPDSWAPPrvLib.${soname}.$fullversion $1/libODPDSWAPPrvLib.${soname}.$majorver
        ln -s libODPCDSWAPPrvLib.${soname}.$fullversion $1/libODPCDSWAPPrvLib.${soname}.$majorver
        ln -s libODPDDSWAPPrvLib.${soname}.$fullversion $1/libODPDDSWAPPrvLib.${soname}.$majorver
   fi
   ln -s libioqhash.${soname}.$fullversion $1/libioqhash.${soname}.$majorver
   # The ODPP-Column Map Hash Interface Name
   ln -s libioqhash.${soname}.$fullversion $1/libioqhash${hybridver}.${soname}
   if [ -e $1/libioqspe.${soname}.$fullversion ]
   then
      ln -s libioqspe.${soname}.$fullversion $1/libioqspe.${soname}.$majorver
   fi
   ln -s libioqldsoralib.${soname}.$fullversion $1/libioqldsoralib.${soname}.$majorver
   ln -s libODPPLoader.${soname}.$fullversion $1/libODPPLoader.${soname}.$majorver
   if [ -e $1/libIOQMODJNI.${soname}.$fullversion ]
   then
      ln -s  libIOQMODJNI.${soname}.$fullversion $1/libIOQMODJNI.$majorver.${soname};
      ln -s  libIOQMODJNI.${soname}.$fullversion $1/libIOQMODJNI.${soname}.$majorver
   fi
   if [ -e $1/libODPPJNI.${soname}.$fullversion ]
   then
      ln -s  libODPPJNI.${soname}.$fullversion $1/libODPPJNI.$majorver.${soname};
      ln -s  libODPPJNI.${soname}.$fullversion $1/libODPPJNI.${soname}.$majorver
   fi
   if [ -e $1/libODPPORAUdf.${soname}.$fullversion ]
   then
      ln -s  libODPPORAUdf.${soname}.$fullversion $1/libODPPORAUdf.${soname}.$majorver;
      ln -s  libODPPORAUdf.${soname}.$fullversion $1/libODPPORAUdf.$majorver.${soname}
   fi
   if [ -e $1/libioqoraudf.${soname}.$fullversion ]
   then
      ln -s  libioqoraudf.${soname}.$fullversion $1/libioqoraudf.${soname}.$majorver;
      ln -s  libioqoraudf.${soname}.$fullversion $1/libioqoraudf.$majorver.${soname}
   fi
   if [ -e $1/libioqdb2udf.${soname}.$fullversion ]
   then
      ln -s  libioqdb2udf.${soname}.$fullversion $1/libioqdb2udf.${soname}.$majorver;
      ln -s  libioqdb2udf.${soname}.$fullversion $1/libioqdb2udf.$majorver.${soname}
   fi
   if [ -e $1/libODPPConvert.${soname}.$fullversion ]
   then
      ln -s  libODPPConvert.${soname}.$fullversion $1/libODPPConvert.${soname}.$majorver
   fi
fi

echo "Creating symbolic links for the ICU ${icuversion}.${icurelease} libraries"

for icufile in ${ICUFILES[@]}
do
   if [ "$platform" = "AIX" ]
    then
       if [ -f $1/${icufile}${icuversion}.${icurelease}.${soname} ]
       then
          ln -s ${icufile}${icuversion}.${icurelease}.${soname} $1/${icufile}${icuversion}.${soname}
       fi
    else
       if [ -f $1/${icufile}.${soname}.${icuversion}.${icurelease} ]
       then
          ln -s ${icufile}.${soname}.${icuversion}.${icurelease} $1/${icufile}.${soname}.${icuversion}
       fi
   fi
done

echo "Setting permissions for the libraries"
# Ensure the files exist before we attempt chmod on those.
ls $1/libicu* >/dev/null 2>&1
[ $? -eq 0 ] && chmod 755 $1/libicu*
chmod 755 $1/libODP*
chmod 755 $1/libioq*
# There is no libIOQ* on RHEL RTL so we check if
# the files exist before we attempt chmod on those.
ls $1/libIOQ* >/dev/null 2>&1
[ $? -eq 0 ] && chmod 755 $1/libIOQ*
echo "Done"
exit 0