/*
    Copyright IBM Corp. 2016 and UNICOM Systems, Inc. 2016

    The following sample of source code ("Sample") is owned by International
    Business Machines Corporation or one of its subsidiaries ("IBM") and/or
    UNICOM Systems, Inc. ("UNICOM") and is copyrighted and licensed, not sold.
    You may use, copy, modify, and distribute the Sample in any form without payment
    to IBM and/or UNICOM, for the purpose of assisting you in the development
    of your applications.

    The Sample code is provided to you on an "AS IS" basis, without warranty of
    any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR
    IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do
    not allow for the exclusion or limitation of implied warranties, so the above
    limitations or exclusions may not apply to you. IBM and UNICOM shall not be liable
    liable for any damages you suffer as a result of using, copying, modifying or
    distributing the Sample, even if IBM and/or UNICOM have been advised of the
    possibility of such damages.
 */

/*******************************************************************************
	This program demonstrates how to perform ODPP masking using a Java program.
	It assumes that the developer is familiar with ODPP and the masking syntax.
	
	To understand this example, first look at the main method in this program.
	It shows how to initialize and instance of ODPP, then it calls several methods
	that demonstrate:
	  - Error handling
	  - Data privacy masking
	  
	NOTES: 
	  - The ODPP Java API only supports wide character data (WVARCHAR_SZ, in ODPP terminology), 
	    which corresponds with the Java String class. If any other data type is to be processed,
	    it must be converted to a Java String. One exception to this rule:  For date fields
	    (notably with the AGE provider) the DATETIME_WVARCHAR type must be used.
	  - While this sample program demonstrates ODPP Affinity, CCN, Email and National ID
	    masking, it is expected to function properly with any other type of data
	    masking supported by the ODPP masking syntax.
 *******************************************************************************/

package com.ibm.odppjavasample;

import com.ibm.nex.odpp.Field;
import com.ibm.nex.odpp.FieldDescriptor;
import com.ibm.nex.odpp.NativeProvider;
import com.ibm.nex.odpp.ODPP;
import com.ibm.nex.odpp.ODPPRuntimeException;
import com.ibm.nex.odpp.ODPPException;
import com.ibm.nex.odpp.RowSet;

public class ODPPJavaSample {

	static private ODPP odpp;
	static private final String sep = "\n--------------\n";
	
	protected static void demoErrorHandling() {
		
	    System.out.println(sep + "Demo of handling error situations......");
	    NativeProvider provider = null;
	    
	    System.out.println("\n\n******Demo with field def missing from mask string:\n");
	    String maskStr = "PROVIDER=NID,switch=us,method=repeatable,val=y";
	    try {
	    	 provider = odpp.createProvider(maskStr);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayCreationException(maskStr, e);
	    }
	    
	    
	    System.out.println("\n\n******Demo of bad method parameter in mask string:\n");
	    maskStr = "PROVIDER=NID,switch=us,method=BadParamForDemo,val=y" +
	    		",flddef1=(name=ssn, len=80, dt=WVARCHAR_SZ)";
	    try {
	    	 provider = odpp.createProvider(maskStr);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayCreationException(maskStr, e);
	    }   
	    
	    // Create the provider properly in order to demo further errors after it's created
	    maskStr = "PROVIDER=NID,switch=us,method=repeatable,val=y" +
	    		",flddef1=(name=ssn, len=80, dt=WVARCHAR_SZ)";
	    try {
	    	 provider = odpp.createProvider(maskStr);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayCreationException(maskStr, e);
	    }   

	    
	    
	    System.out.println("\n\n******Demo of attempt to mask invalid NID:");
	    FieldDescriptor[] fieldDescriptors = new FieldDescriptor[1];
	    fieldDescriptors[0] = new FieldDescriptor("ssn", provider);  // Must match the name in flddef in mask string
	    
	    // Set up a rowset with one row, to demo masking a single row
	    RowSet rowSet1 = odpp.createRowSet(1, fieldDescriptors);
	    
	    Field field1 = rowSet1.getRow(0).getField(0);
	    String testFld = "badniddemo"; 
	    field1.setValue(testFld);
	    
	    // Call provider to mask the data
	    try {
	    	provider.service(rowSet1);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayServiceException(e);
	    	return;
	    }
	    System.out.println("\nOld ssn: " + testFld + "\nNew ssn: " + field1.getValue() + getErrInfo(field1));
	
	    // Shut down the provider
	    try {
	       provider.shutdown();
	    }
	    catch (ODPPRuntimeException e) {
	       displayProviderShutdownException(e);
	    }
	}

	protected static void maskAFF() {
		
	    System.out.println(sep + "Demo of single call to AFF provider, with one row:");
	
	    // Initialize the provider and field descriptors
	    NativeProvider provider;
	    String maskStr = "PROVIDER=AFF,method=repeatable,copy=(1,4)" +
	    		",flddef1=(name=acctnbr, len=80, dt=WVARCHAR_SZ)";
	    try {
	    	 provider = odpp.createProvider(maskStr);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayCreationException(maskStr, e);
	    	return;
	    }
	    FieldDescriptor[] fieldDescriptors = new FieldDescriptor[1];
	    fieldDescriptors[0] = new FieldDescriptor("acctnbr", provider);  // Must match the name in flddef in mask string
	    
	    // Set up a rowset with one row, to demo masking a single row
	    RowSet rowSet1 = odpp.createRowSet(1, fieldDescriptors);
	    
	    Field field1 = rowSet1.getRow(0).getField(0);
	    String testFld = "acct12345";
	    field1.setValue(testFld);
	    
	    // Call provider to mask the data
	    try {
	    	provider.service(rowSet1);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayServiceException(e);
	    	return;
	    }
	    System.out.println("\nOld value: " + testFld + "\nNew value: " + field1.getValue() + getErrInfo(field1));
	
	    // Shut down the provider
	    try {
		   provider.shutdown();
		}
	    catch (ODPPRuntimeException e) {
		   displayProviderShutdownException(e);
		}
	}
	
    protected static void maskCCN() {
    	
        System.out.println(sep + "Demo of multiple calls to CCN provider, with one row per call:");

        // Initialize the provider and field descriptors
        NativeProvider provider;
        String maskStr = "PROVIDER=CCN,flddef1=(name=ccn, len=80, dt=WVARCHAR_SZ)"; 
        try {
           provider = odpp.createProvider(maskStr);
        }
        catch (ODPPRuntimeException e) {
           displayCreationException(maskStr, e);
           return;
        }
        FieldDescriptor[] fieldDescriptors = new FieldDescriptor[1];
        fieldDescriptors[0] = new FieldDescriptor("ccn", provider);  // Must match the name in flddef in mask string
        
        // Set up a rowset with one row, to demo masking a single CCN per call to the provider
        RowSet rowSet1 = odpp.createRowSet(1, fieldDescriptors);
        
        Field field1 = rowSet1.getRow(0).getField(0);
        String testCCN = "4571444025760464";
        field1.setValue(testCCN);
        
        // Call provider to mask the data
        try {
        	provider.service(rowSet1);
        }
        catch (ODPPRuntimeException e) {
        	displayServiceException(e);
        	return;
        }
        System.out.println("\nOld CCN: " + testCCN + "\nNew CCN: " + field1.getValue() + getErrInfo(field1));
        
        // Mask another CCN
        testCCN = "30000019965395";
        field1.setValue(testCCN);
        provider.service(rowSet1);  // try/catch not done here to keep the sample program compact
        System.out.println("\nOld CCN: " + testCCN + "\nNew CCN: " + field1.getValue() + getErrInfo(field1));
        
        
        // Set up a rowset with multiple rows to demo masking multiple CCN's per call to the provider
        System.out.println(sep + "Demo of one call to CCN provider, with multiple rows:");
        String[] ccns = {"4571444025760464", "30000019965395", "30404261160731"};
        int nRows = ccns.length;
        RowSet rowSet2 = odpp.createRowSet(nRows, fieldDescriptors);
        for (int i = 0; i < nRows; i++) {
        	Field field2 = rowSet2.getRow(i).getField(0);
        	field2.setValue(ccns[i]);
        }
        provider.service(rowSet2);  // try/catch not done here to keep the sample program compact
        for (int i = 0; i < nRows; i++) {
        	System.out.println("\nOld CCN: " + ccns[i] + "\nNew CCN: " + rowSet2.getRow(i).getField(0).getValue() + getErrInfo(rowSet2.getRow(i).getField(0)));	
        }
        
        // Shut down the provider
	    try {
		   provider.shutdown();
		}
	    catch (ODPPRuntimeException e) {
		   displayProviderShutdownException(e);
		}
    }

    protected static void maskEML() {
    	
        System.out.println(sep + "Demo of single call to EML provider, with one row, and multiple flddefs:");

        // Initialize the provider and field descriptors
        NativeProvider provider;
        String maskStr = "PROVIDER=EML,parts=\"(2,3)\",sep=dot" +
        		",flddef1=(name=eml, len=80, dt=WVARCHAR_SZ)" +
        		",flddef2=(name=firstname, len=80, dt=WVARCHAR_SZ)" +
        		",flddef3=(name=lastname,  len=80, dt=WVARCHAR_SZ)";
        try {
        	 provider = odpp.createProvider(maskStr);
        }
        catch (ODPPRuntimeException e) {
        	displayCreationException(maskStr, e);
        	return;
        }
        FieldDescriptor[] fieldDescriptors = new FieldDescriptor[3];
        fieldDescriptors[0] = new FieldDescriptor("eml", provider);         // - Must match the names in flddef in mask string
        fieldDescriptors[1] = new FieldDescriptor("firstname", provider);   // - And must be in the same order as flddef1, flddef2, flddef3
        fieldDescriptors[2] = new FieldDescriptor("lastname", provider);    //   in masking string
        
        // Set up a rowset with one row, to demo masking a single EML
        RowSet rowSet1 = odpp.createRowSet(1, fieldDescriptors);
        
        Field field1 = rowSet1.getRow(0).getField(0);
        String testEml = "abc.def@ghi.com";
        field1.setValue(testEml);
        
        Field field2 = rowSet1.getRow(0).getField(1);
        String testFirstname = "newfirstname";
        field2.setValue(testFirstname);
        
        Field field3 = rowSet1.getRow(0).getField(2);
        String testLastname = "newlastname";
        field3.setValue(testLastname);
        
        // Call provider to mask the data
        try {
        	provider.service(rowSet1);
        }
        catch (ODPPRuntimeException e) {
        	displayServiceException(e);
        	return;
        }
        System.out.println("\nOld Email: " + testEml + "\nNew Email: " + field1.getValue() + getErrInfo(field1));
   
        // Shut down the provider
	    try {
		   provider.shutdown();
		}
	    catch (ODPPRuntimeException e) {
		   displayProviderShutdownException(e);
		}
    }
    
	protected static void maskHASH() {
		
	    System.out.println(sep + "Demo of single call to HASH provider, with one row:");
	
	    // Initialize the provider and field descriptors
	    NativeProvider provider;
	    String maskStr = "pro=hash, case=up, remove=\"xy:\",sep=\";\", hashmax=555555" +
	                     ", dstfld=destcolumn, flddef1=(name=acctnbr,dt=WVARCHAR_SZ), cp=1252";

	    try {
	    	 provider = odpp.createProvider(maskStr);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayCreationException(maskStr, e);
	    	return;
	    }
	    FieldDescriptor[] fieldDescriptors = new FieldDescriptor[2];
	    fieldDescriptors[0] = new FieldDescriptor("acctnbr", provider);     // Must match the names in flddef in mask string
	    fieldDescriptors[1] = new FieldDescriptor("destcolumn", provider);  // *
	    
	    // Set up a rowset with one row, to demo masking a single row
	    RowSet rowSet1 = odpp.createRowSet(1, fieldDescriptors);
	    
	    Field field1 = rowSet1.getRow(0).getField(0);
	    String testFld = "abcdefg";
	    field1.setValue(testFld);
	    
	    Field field2 = rowSet1.getRow(0).getField(1);
	    String destFld = new String();
	    field2.setValue(destFld);
	        
	        
	    // Call provider to mask the data
	    try {
	    	provider.service(rowSet1);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayServiceException(e);
	    	return;
	    }
	    System.out.println("\nOld value: " + testFld + "\nNew value: " + field2.getValue() + getErrInfo(field1));
	
	    // Shut down the provider
	    try {
		   provider.shutdown();
		}
	    catch (ODPPRuntimeException e) {
		   displayProviderShutdownException(e);
		}
	}
	
	protected static void maskNID() {
		
	    System.out.println(sep + "Demo of single call to NID provider, with one row:");
	
	    // Initialize the provider and field descriptors
	    NativeProvider provider;
	    String maskStr = "PROVIDER=NID,switch=us,method=repeatable,val=y" +
	    		",flddef1=(name=ssn, len=80, dt=WVARCHAR_SZ)";
	    try {
	    	 provider = odpp.createProvider(maskStr);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayCreationException(maskStr, e);
	    	return; 
	    }
	    FieldDescriptor[] fieldDescriptors = new FieldDescriptor[1];
	    fieldDescriptors[0] = new FieldDescriptor("ssn", provider);  // Must match the name in flddef in mask string
	    
	    // Set up a rowset with one row, to demo masking a single row
	    RowSet rowSet1 = odpp.createRowSet(1, fieldDescriptors);
	    
	    Field field1 = rowSet1.getRow(0).getField(0);
	    String testFld = "381275346";
	    field1.setValue(testFld);
	    
	    // Call provider to mask the data
	    try {
	    	provider.service(rowSet1);
	    }
	    catch (ODPPRuntimeException e) {
	    	displayServiceException(e);
	    	return;
	    }
	    System.out.println("\nOld ssn: " + testFld + "\nNew ssn: " + field1.getValue() + getErrInfo(field1));
	
	    // Shut down the provider
	    try {
		   provider.shutdown();
		}
	    catch (ODPPRuntimeException e) {
		   displayProviderShutdownException(e);
		}
	}
	
    
	private static String getErrInfo(Field field1) {
		int errCode = field1.getErrorCode();
		if  (errCode == 0) {
			return "";
		}
		else {
			String errInfo = "\n***Error code: " + errCode;
			String msg = field1.getErrorMessage();
			if (msg != null) {
				errInfo = errInfo + '\n' + msg;
			}
			return errInfo;
		}
	}

	protected static void displayCreationException(String maskStr, ODPPRuntimeException e) {
		String msg = "ERROR while creating provider:\nMask String: " + maskStr + '\n' + e.getMessages()[1];
		System.out.println(msg);
	}
	
	protected static void displayServiceException(ODPPRuntimeException e) {
		String msg;
		String[]  messages = e.getMessages();
		if (messages.length > 1)
			msg = "ERROR in masking service: " + messages[1];
		else
			msg = "ERROR in masking service: " + messages[0]; // Will happen on JNI error
		System.out.println(msg);
	}
	
	protected static void displayProviderShutdownException(ODPPRuntimeException e) {
		String msg;
		String[]  messages = e.getMessages();
		if (messages.length > 1)
			msg = "ERROR in provider shutdown: " + messages[1];
		else
			msg = "ERROR in provider shutdown: " + messages[0]; // Will happen on JNI error
		System.out.println(msg);
	}
	
	public static void main(String[] args) {
		
		// Initialize the system
        odpp = new ODPP(); 
        String jniModule = "ODPPJNI.11.7"; // Version number must match ODPP installation
        odpp.setJNILIBNAME(jniModule);  
        try {
        	odpp.initialize();
        }
        catch (UnsatisfiedLinkError e) {
        	System.out.println("ERROR could not load JNI Module, " + jniModule);
        	System.out.println(       "This might be due to a mismatch of bitness of the ODPP library and the the bitness of the JRE.");
        	System.out.println("      If you are using a 64-bit JRE, then, then please ensure you are using a 64-bit Modapp/ODPP");
        	System.out.println("      If you are using a 32-bit JRE, then, then please ensure you are using a 32-bit Modapp/ODPP");
        	return;
        }
        catch (ODPPException e) {
        	System.out.println("ERROR while initializing ODPP framework.  Check that the license file is present and not expired.\nODPP ReturnValue:" + e.getCode());
        	return;
        }
        catch (ODPPRuntimeException e) {
        	String messages[] = e.getMessages();
        	for (String msg : messages) {
        	  System.out.println(msg);
        	}
        	return;
        }
        
        // Demonstrate handling of various types of errors
        demoErrorHandling();
        
        // Demonstrate various types of data masking
        maskAFF();
		maskCCN();
		maskEML();
		maskHASH();
		maskNID();
		
		// Close down the system
        try {
           odpp.shutdown();
        }
        catch (ODPPException e) {
          System.out.println("ERROR while shutting down ODPP framework.\nODPP ReturnValue:" + e.getCode());
        }
        System.out.println(sep + "End of processing");
	}


}
